@@ -61,7 +61,7 @@ gs_review_row_refresh (GsReviewRow *row)
 	GsReviewRowPrivate *priv = gs_review_row_get_instance_private (row);
 	const gchar *reviewer;
 	GDateTime *date;
-	g_autofree gchar *text;
+	g_autofree gchar *text = NULL;
 
 	gs_star_widget_set_rating (GS_STAR_WIDGET (priv->stars),
 				   gs_review_get_rating (priv->review));
@@ -183,11 +183,59 @@ gs_review_row_button_clicked_downvote_cb (GtkButton *button, GsReviewRow *row)
 		       GS_REVIEW_ACTION_DOWNVOTE);
 }
 
+static void
+gs_review_row_confirm_cb (GtkDialog *dialog, gint response_id, GsReviewRow *row)
+{
+	if (response_id == GTK_RESPONSE_YES) {
+		g_signal_emit (row, signals[SIGNAL_BUTTON_CLICKED], 0,
+			       GS_REVIEW_ACTION_REPORT);
+	}
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
 static void
 gs_review_row_button_clicked_report_cb (GtkButton *button, GsReviewRow *row)
 {
-	g_signal_emit (row, signals[SIGNAL_BUTTON_CLICKED], 0,
-		       GS_REVIEW_ACTION_REPORT);
+	GtkWidget *dialog;
+	GtkWidget *toplevel;
+	GtkWidget *widget;
+	g_autoptr(GString) str = NULL;
+
+	str = g_string_new ("");
+
+	/* TRANSLATORS: we explain what the action is going to do */
+	g_string_append (str, _("You can report reviews for abusive, rude, or "
+				"discriminatory behavior."));
+	g_string_append (str, " ");
+
+	/* TRANSLATORS: we ask the user if they really want to do this */
+	g_string_append (str, _("Once reported, a review will be hidden until "
+				"it has been checked by an administrator."));
+
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (button));
+	dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
+					 GTK_DIALOG_MODAL |
+					 GTK_DIALOG_DESTROY_WITH_PARENT |
+					 GTK_DIALOG_USE_HEADER_BAR,
+					 GTK_MESSAGE_QUESTION,
+					 GTK_BUTTONS_CANCEL,
+					 "%s",
+					 /* TRANSLATORS: window title when
+					  * reporting a user-submitted review
+					  * for moderation */
+					 _("Report Review?"));
+	widget = gtk_dialog_add_button (GTK_DIALOG (dialog),
+					/* TRANSLATORS: button text when
+					 * sending a review for moderation */
+					_("Report"),
+					GTK_RESPONSE_YES);
+	gtk_style_context_add_class (gtk_widget_get_style_context (widget),
+				     "destructive-action");
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  "%s", str->str);
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (gs_review_row_confirm_cb), row);
+	gtk_window_present (GTK_WINDOW (dialog));
 }
 
 static void
