@@ -953,6 +953,8 @@ gs_shell_updates_offline_update_cb (GsPluginLoader *plugin_loader,
 	g_autoptr(GDBusConnection) bus = NULL;
 	g_autoptr(GError) error = NULL;
 
+	gtk_widget_set_sensitive (GTK_WIDGET (self->button_update_all), TRUE);
+
 	/* get the results */
 	if (!gs_plugin_loader_offline_update_finish (plugin_loader, res, &error)) {
 		g_warning ("Failed to trigger offline update: %s", error->message);
@@ -979,6 +981,8 @@ gs_shell_updates_button_update_all_cb (GtkButton      *button,
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GList) apps = NULL;
 
+	gtk_widget_set_sensitive (GTK_WIDGET (self->button_update_all), FALSE);
+
 	/* do the offline update */
 	apps = gs_update_list_get_apps (GS_UPDATE_LIST (self->list_box_updates));
 	gs_plugin_loader_offline_update_async (self->plugin_loader,
@@ -1016,17 +1020,11 @@ upgrade_download_finished_cb (GObject *source,
 	g_autoptr(GsPageHelper) helper = (GsPageHelper *) user_data;
 
 	if (!gs_plugin_loader_app_action_finish (plugin_loader, res, &error)) {
-		g_warning ("failed to upgrade-download: %s", error->message);
 		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 			return;
-		gs_app_notify_failed_modal (helper->app,
-					    gs_shell_get_window (helper->self->shell),
-					    GS_PLUGIN_LOADER_ACTION_UPGRADE_DOWNLOAD,
-					    error);
-		return;
+		g_warning ("failed to upgrade-download: %s", error->message);
 	}
 
-	/* non-fatal error */
 	last_error = gs_app_get_last_error (helper->app);
 	if (last_error != NULL) {
 		g_warning ("failed to upgrade-download %s: %s",
@@ -1166,6 +1164,29 @@ gs_shell_updates_upgrade_help_cb (GsUpgradeBanner *upgrade_banner,
 		g_warning ("failed to open %s: %s", uri, error->message);
 }
 
+static void
+gs_shell_updates_invalidate_downloaded_upgrade (GsShellUpdates *self)
+{
+	GsApp *app;
+	app = gs_upgrade_banner_get_app (GS_UPGRADE_BANNER (self->upgrade_banner));
+	if (app == NULL)
+		return;
+	if (gs_app_get_state (app) != AS_APP_STATE_UPDATABLE)
+		return;
+	gs_app_set_state (app, AS_APP_STATE_AVAILABLE);
+	g_debug ("resetting %s to AVAILABLE as the updates have changed",
+		 gs_app_get_id (app));
+}
+
+static void
+gs_shell_updates_changed_cb (GsPluginLoader *plugin_loader,
+			     GsShellUpdates *self)
+{
+	/* if we do a live update and the upgrade is waiting to be deployed
+	 * then make sure all new packages are downloaded */
+	gs_shell_updates_invalidate_downloaded_upgrade (self);
+}
+
 /**
  * gs_shell_updates_status_changed_cb:
  **/
@@ -1175,6 +1196,20 @@ gs_shell_updates_status_changed_cb (GsPluginLoader *plugin_loader,
 				    GsPluginStatus status,
 				    GsShellUpdates *self)
 {
+	switch (status) {
+	case GS_PLUGIN_STATUS_INSTALLING:
+	case GS_PLUGIN_STATUS_REMOVING:
+		if (gs_app_get_kind (app) != AS_APP_KIND_OS_UPGRADE &&
+		    gs_app_get_id (app) != NULL) {
+			/* if we do a install or remove then make sure all new
+			 * packages are downloaded */
+			gs_shell_updates_invalidate_downloaded_upgrade (self);
+		}
+		break;
+	default:
+		break;
+	}
+
 	self->last_status = status;
 	gs_shell_updates_update_ui_state (self);
 }
@@ -1230,6 +1265,9 @@ gs_shell_updates_setup (GsShellUpdates *self,
 	g_signal_connect (self->plugin_loader, "status-changed",
 			  G_CALLBACK (gs_shell_updates_status_changed_cb),
 			  self);
+	g_signal_connect (self->plugin_loader, "updates-changed",
+			  G_CALLBACK (gs_shell_updates_changed_cb),
+			  self);
 	self->builder = g_object_ref (builder);
 	self->cancellable = g_object_ref (cancellable);
 
