@@ -222,6 +222,10 @@ get_upgrades_finished_cb (GObject *object,
 		return;
 	}
 
+	/* do not show if gnome-software is already open */
+	if (gs_application_has_active_window (GS_APPLICATION (monitor->application)))
+		return;
+
 	/* just get the first result : FIXME, do we sort these by date? */
 	app = GS_APP (apps->data);
 
@@ -333,7 +337,8 @@ check_updates (GsUpdateMonitor *monitor)
 	g_debug ("Refreshing cache");
 	gs_plugin_loader_refresh_async (monitor->plugin_loader,
 					60 * 60 * 24,
-					GS_PLUGIN_REFRESH_FLAGS_UPDATES,
+					GS_PLUGIN_REFRESH_FLAGS_METADATA |
+					GS_PLUGIN_REFRESH_FLAGS_PAYLOAD,
 					monitor->cancellable,
 					refresh_cache_finished_cb,
 					monitor);
@@ -428,7 +433,6 @@ get_updates_historical_cb (GObject *object, GAsyncResult *res, gpointer data)
 			notification = g_notification_new (_("Software Updates Failed"));
 			/* TRANSLATORS: message when we offline updates have failed */
 			g_notification_set_body (notification, _("An important OS update failed to be installed."));
-			g_application_send_notification (monitor->application, "offline-updates", notification);
 			g_notification_add_button (notification, _("Show Details"), "app.show-offline-update-error");
 			g_notification_set_default_action (notification, "app.show-offline-update-error");
 			g_application_send_notification (monitor->application, "offline-updates", notification);
@@ -491,104 +495,12 @@ cleanup_notifications_cb (gpointer user_data)
 	return G_SOURCE_REMOVE;
 }
 
-static void
-do_not_expand (GtkWidget *child, gpointer data)
-{
-	gtk_container_child_set (GTK_CONTAINER (gtk_widget_get_parent (child)),
-				 child, "expand", FALSE, "fill", FALSE, NULL);
-}
-
-static gboolean
-unset_focus (GtkWidget *widget, GdkEvent *event, gpointer data)
-{
-	if (GTK_IS_WINDOW (widget))
-		gtk_window_set_focus(GTK_WINDOW (widget), NULL);
-	return FALSE;
-}
-
-/**
- * insert_details_widget:
- * @dialog: the message dialog where the widget will be inserted
- * @details: (allow-none): the detailed message text to display
- *
- * Inserts a widget displaying the detailed message into the message dialog.
- * Does nothing if @details is %NULL so it is safe to call this function
- * without checking if there is anything to display.
- */
-static void
-insert_details_widget (GtkMessageDialog *dialog, const gchar *details)
-{
-	GtkWidget *message_area, *sw, *label;
-	GtkWidget *box, *tv;
-	GtkTextBuffer *buffer;
-	GList *children;
-	g_autoptr(GString) msg = NULL;
-
-	if (!details)
-		return;
-	g_return_if_fail (dialog != NULL);
-	gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
-
-	msg = g_string_new ("");
-	g_string_append_printf (msg, "%s\n\n%s",
-				/* TRANSLATORS: these are show_detailed_error messages from the
-				 * package manager no mortal is supposed to understand,
-				 * but google might know what they mean */
-				_("Detailed errors from the package manager follow:"),
-				details);
-
-	message_area = gtk_message_dialog_get_message_area (dialog);
-	g_assert (GTK_IS_BOX (message_area));
-	/* make the hbox expand */
-	box = gtk_widget_get_parent (message_area);
-	gtk_container_child_set (GTK_CONTAINER (gtk_widget_get_parent (box)), box,
-				 "expand", TRUE, "fill", TRUE, NULL);
-	/* make the labels not expand */
-	gtk_container_foreach (GTK_CONTAINER (message_area), do_not_expand, NULL);
-
-	/* Find the secondary label and set its width_chars.   */
-	/* Otherwise the label will tend to expand vertically. */
-	children = gtk_container_get_children (GTK_CONTAINER (message_area));
-	if (children && children->next && GTK_IS_LABEL (children->next->data)) {
-		gtk_label_set_width_chars (GTK_LABEL (children->next->data), 40);
-	}
-
-	label = gtk_label_new (_("Details"));
-	gtk_widget_set_halign (label, GTK_ALIGN_START);
-	gtk_widget_set_visible (label, TRUE);
-	gtk_box_pack_start (GTK_BOX (message_area), label, FALSE, FALSE, 0);
-
-	sw = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
-					     GTK_SHADOW_IN);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-					GTK_POLICY_NEVER,
-					GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_min_content_height (GTK_SCROLLED_WINDOW (sw), 150);
-	gtk_widget_set_visible (sw, TRUE);
-
-	tv = gtk_text_view_new ();
-	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tv));
-	gtk_text_view_set_editable (GTK_TEXT_VIEW (tv), FALSE);
-	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (tv), GTK_WRAP_WORD);
-	gtk_style_context_add_class (gtk_widget_get_style_context (tv),
-	                             "update-failed-details");
-	gtk_text_buffer_set_text (buffer, msg->str, -1);
-	gtk_widget_set_visible (tv, TRUE);
-
-	gtk_container_add (GTK_CONTAINER (sw), tv);
-	gtk_box_pack_end (GTK_BOX (message_area), sw, TRUE, TRUE, 0);
-
-	g_signal_connect (dialog, "map-event", G_CALLBACK (unset_focus), NULL);
-}
-
 void
 gs_update_monitor_show_error (GsUpdateMonitor *monitor, GsShell *shell)
 {
 	const gchar *title;
 	const gchar *msg;
 	gboolean show_detailed_error;
-	GtkWidget *dialog;
 
 	/* can this happen in reality? */
 	if (monitor->last_offline_error == NULL)
@@ -636,21 +548,10 @@ gs_update_monitor_show_error (GsUpdateMonitor *monitor, GsShell *shell)
 		break;
 	}
 
-	dialog = gtk_message_dialog_new_with_markup (gs_shell_get_window (shell),
-					 0,
-					 GTK_MESSAGE_INFO,
-					 GTK_BUTTONS_CLOSE,
-					 "<big><b>%s</b></big>", title);
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  "%s", msg);
-	if (show_detailed_error) {
-		insert_details_widget (GTK_MESSAGE_DIALOG (dialog),
-				       monitor->last_offline_error->message);
-	}
-	g_signal_connect_swapped (dialog, "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  dialog);
-	gtk_widget_show (dialog);
+	gs_utils_show_error_dialog (gs_shell_get_window (shell),
+	                            title,
+	                            msg,
+	                            show_detailed_error ? monitor->last_offline_error->message : NULL);
 }
 
 static void
@@ -753,8 +654,10 @@ GPermission *
 gs_update_monitor_permission_get (void)
 {
 	static GPermission *permission = NULL;
+#ifdef HAVE_PACKAGEKIT
 	if (permission == NULL)
 		permission = gs_utils_get_permission ("org.freedesktop.packagekit.trigger-offline-update");
+#endif
 	return permission;
 }
 
