@@ -108,6 +108,7 @@ gs_plugin_destroy (GsPlugin *plugin)
 
 
 typedef struct {
+	GsApp		*app;
 	GsPlugin	*plugin;
 	AsProfileTask	*ptask;
 	gchar		*profile_id;
@@ -146,7 +147,7 @@ gs_plugin_packagekit_progress_cb (PkProgress *progress,
 
 	plugin_status = packagekit_status_enum_to_plugin_status (status);
 	if (plugin_status != GS_PLUGIN_STATUS_UNKNOWN)
-		gs_plugin_status_update (plugin, NULL, plugin_status);
+		gs_plugin_status_update (plugin, data->app, plugin_status);
 }
 
 static void
@@ -284,6 +285,7 @@ gs_plugin_packagekit_resolve_packages (GsPlugin *plugin,
 	}
 	g_ptr_array_add (package_ids, NULL);
 
+	data.app = NULL;
 	data.plugin = plugin;
 	data.ptask = NULL;
 	data.profile_id = NULL;
@@ -311,7 +313,7 @@ gs_plugin_packagekit_resolve_packages (GsPlugin *plugin,
 
 static gboolean
 gs_plugin_packagekit_refine_from_desktop (GsPlugin *plugin,
-					  GsApp	 *app,
+					  GsApp *app,
 					  const gchar *filename,
 					  GCancellable *cancellable,
 					  GError **error)
@@ -321,6 +323,7 @@ gs_plugin_packagekit_refine_from_desktop (GsPlugin *plugin,
 	g_autoptr(PkResults) results = NULL;
 	g_autoptr(GPtrArray) packages = NULL;
 
+	data.app = app;
 	data.plugin = plugin;
 	data.ptask = NULL;
 	data.profile_id = g_path_get_basename (filename);
@@ -403,6 +406,7 @@ gs_plugin_packagekit_refine_updatedetails (GsPlugin *plugin,
 		package_ids[i++] = package_id;
 	}
 
+	data.app = NULL;
 	data.plugin = plugin;
 	data.ptask = NULL;
 	data.profile_id = NULL;
@@ -541,6 +545,7 @@ gs_plugin_packagekit_refine_details (GsPlugin *plugin,
 	}
 	g_ptr_array_add (package_ids, NULL);
 
+	data.app = NULL;
 	data.plugin = plugin;
 	data.ptask = NULL;
 	data.profile_id = g_strjoinv (",", (gchar **) package_ids->pdata);
@@ -580,6 +585,7 @@ gs_plugin_packagekit_refine_update_urgency (GsPlugin *plugin,
 	g_autoptr(PkPackageSack) sack = NULL;
 	g_autoptr(PkResults) results = NULL;
 
+	data.app = NULL;
 	data.plugin = plugin;
 	data.ptask = NULL;
 	data.profile_id = NULL;
@@ -774,6 +780,7 @@ gs_plugin_packagekit_refine_distro_upgrade (GsPlugin *plugin,
 	g_autoptr(PkResults) results = NULL;
 	g_autoptr(GsAppList) list = NULL;
 
+	data.app = app;
 	data.plugin = plugin;
 	data.ptask = NULL;
 	data.profile_id = NULL;
